# -*- coding: utf-8 -*-

import pandas as pd

from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.core.manager import DatabaseDataManager


class ChatDataManager(DatabaseDataManager):
    def __init__(self, client=None) -> None:
        super().__init__(client, create_logger(__name__, 'chat_data_manager.log'))

    def _make_query_sql(self, conditions=None):

        if conditions:
            conditions = ['doctor_id IS NOT NULL'] + conditions
        else:
            conditions = ['doctor_id IS NOT NULL']
        
        condition_sql = ' WHERE ' + ' AND '.join(conditions)

        select_fields = ['dt', 'uid', 'doctor_id', 'user_to_doctor', 'doctor_to_user']
        sql = 'SELECT {} FROM dws.dws_user_chat_assistant_doctor_day'.format(', '.join(select_fields))
        sql += condition_sql
        return sql

    def update_data(self):
        """ 从数据库中拉取最新订单数据并保存 """
        sql = self._make_query_sql()
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_csv_data(df, 'all_chat_info.csv')
        return df

    def update_test_data(self, conditions):
        """ 从数据库中拉取指定条件订单用于测试 """

        sql = self._make_query_sql(conditions)
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_csv_data(df, 'test_chat_info.csv')

    def load_raw_data(self):
        return self.load_csv_data('all_chat_info.csv')

    def load_test_data(self):
        return self.load_csv_data('test_chat_info.csv')



if __name__ == '__main__':
    from ydl_ai_recommender.src.data.mysql_client import MySQLClient
    from ydl_ai_recommender.src.utils import get_conf_path


    client = MySQLClient.create_from_config_file(get_conf_path())
    manager = ChatDataManager(client)
    # manager.update_data()
    manager.make_index()