# -*- coding: utf-8 -*-

import json
import random
from collections import Counter

from ydl_ai_recommender.src.core.manager import (
    CounselorProfileManager,
    UserEventManager,
)
from ydl_ai_recommender.src.core.profile import encode_user_profile
from ydl_ai_recommender.src.core.counselor_profile import encode_counselor_profile
from ydl_ai_recommender.src.core.rank.ranker import Ranker


ranker = Ranker(is_use_db=False)


def make_data(start_date, end_date):
    
    # 平台用户数量太多，仅处理与本次任务相关的用户
    manager = UserEventManager()
    user_profiles = manager.load_xlsx_data(f'expose_click_user_{start_date}_{end_date}.xlsx')
    user_embeddings = {}
    for _, profile in user_profiles.iterrows():
        user_embeddings[str(profile['uid'])] = encode_user_profile(profile)

    counselor_profile_manager = CounselorProfileManager()
    counselor_profiles = counselor_profile_manager.load_xlsx_data('all_counselor_profile.xlsx')

    counselor_embeddings = {}
    for _, profile in counselor_profiles.iterrows():
        counselor_embeddings[str(profile['doctor_id'])] = encode_counselor_profile(profile)

    # 曝光点击数据
    test_data = []
    expost_click = manager.load_data(start_date=start_date, end_date=end_date)
    for _, profile in expost_click.iterrows():
        uid = str(profile['user_ids'])
        if ',' in uid:
            continue

        if uid not in user_embeddings:
            u_emb = ranker.default_user_embedding()
        else:
            u_emb = user_embeddings[uid]

        expose_ids = set(str(profile['ae_doctor_ids']).split(','))
        click_ids = set(str(profile['doctor_card_doctor_ids']).split(',') + str(profile['private_chat_doctor_ids']).split(','))
        uninterested_ids = list(expose_ids - click_ids)

        for _id in click_ids:
            click_id = _id
            if _id in user_embeddings:
                click_emb = counselor_embeddings[_id]
            else:
                click_emb = ranker.default_counselor_embedding()

            c_embs = [click_emb]
            c_ids = [click_id]

            for _id in random.choices(uninterested_ids, k=min(19, len(uninterested_ids))):
                if _id not in counselor_embeddings:
                    continue
                c_embs.append(counselor_embeddings[_id])
                c_ids.append(_id)

            if len(c_embs) < 2:
                continue

            test_data.append([
                c_ids,
                u_emb,
                c_embs,
            ])
    print('test data row: ', len(test_data))
    return test_data
    

def test():
    start_date = '2023-02-01'
    end_date = '2023-02-10'
    test_data = make_data(start_date, end_date)

    result_detail = []
    for [c_ids, u_emb, c_embs] in test_data:
        rank_result = ranker._rank(c_ids, u_emb, c_embs)

        for i, result in enumerate(rank_result):
            if result['counselor'] == c_ids[0]:
                result_detail.append(i)
                break
        else:
            result_detail.append(-1)

    cnter = Counter(result_detail)
    acc_detail = {}
    for key, val in cnter.most_common():
        acc_detail[key] = val
        # print(key, val, val/len(result_detail))

    print('==' * 20)
    cnt = 0
    for n in range(10):
        if n in (1, 3, 5, 10):
            print('top{:2}\t{:5}\t{:.2%}'.format(n, cnt, cnt / len(result_detail)))
        cnt += acc_detail[n]


if __name__ == '__main__':
    test()